@@ -31,6 +31,9 @@ class ULAddObject(ULActionNode):
             return
         life = self.get_input(self.life)
         name = self.get_input(self.name)
+        name = getattr(name, 'name', name)
+        print('################')
+        print(name)
         full_copy = self.get_input(self.full_copy)
         self._set_ready()
         reference = self.get_input(self.reference)
