@@ -1,3 +1,5 @@
+from uplogic.animation import ULActionSystem
+from uplogic.data import GlobalDB
 from uplogic.nodes import ULActionNode
 from uplogic.nodes import ULOutSocket
 from uplogic.utils import is_invalid
@@ -13,7 +15,15 @@ class ULStopAction(ULActionNode):
         self.game_object = None
         self.action_layer = None
         self.done = None
+        self.act_system = self.get_act_system()
         self.OUT = ULOutSocket(self, self.get_done)
+        
+    def get_act_system(self):
+        act_systems = GlobalDB.retrieve('uplogic.animation')
+        if act_systems.check('default'):
+            return act_systems.get('default')
+        else:
+            return ULActionSystem('default')
 
     def get_done(self):
         return self.done
@@ -33,5 +43,8 @@ class ULStopAction(ULActionNode):
             return
         if is_invalid(action_layer):
             return
-        game_object.stopAction(action_layer)
+        
+        action = self.act_system.get_layer(game_object, action_layer)
+        if action is not None:
+            action.stop()
         self.done = True
