@@ -1,13 +1,32 @@
 '''TODO: Documentation
 '''
-
+from .curves import ULCurve
+from .curves import create_curve  # noqa
 from .curves import set_curve_points  # noqa
-from .raycasting import raycast
-from .raycasting import raycast_projectile
-
-from bge import logic, render
+from .nodetrees import get_geom_socket  # noqa
+from .nodetrees import get_material_socket  # noqa
+from .nodetrees import get_world_socket  # noqa
+from .nodetrees import modify_geom_socket  # noqa
+from .nodetrees import modify_material_socket  # noqa
+from .nodetrees import modify_world_socket  # noqa
+from .nodetrees import set_geom_socket  # noqa
+from .nodetrees import set_material_socket  # noqa
+from .nodetrees import set_world_socket  # noqa
+from .lights import ULLight
+from .lights import make_unique_light
+from .raycasting import raycast  # noqa
+from .raycasting import raycast_camera  # noqa
+from .raycasting import raycast_face  # noqa
+from .raycasting import raycast_projectile  # noqa
+from .scene import set_scene  # noqa
+from .visuals import draw_cube
+from .visuals import draw_line
+from .visuals import draw_box
+from bge import logic
 from bge.types import KX_GameObject as GameObject
+from mathutils import Matrix
 from mathutils import Vector
+
 import bpy
 import json
 import math
@@ -36,7 +55,6 @@ STATUS_INVALID = _Status("INVALID")
 VEHICLE = '.ulvehicleconst'
 SHIP = '.ulshipconst'
 FLOATSAM = '.ulfloatsamconst'
-
 WATER = '.ulwater'
 
 
@@ -50,6 +68,18 @@ LOGIC_OPERATORS = [
 ]
 
 
+OPERATORS = {
+    "ADD": operator.add,
+    "DIV": operator.truediv,
+    "MUL": operator.mul,
+    "SUB": operator.sub,
+    'FDIV': operator.floordiv,
+    'MATMUL': operator.matmul,
+    'MOD': operator.mod,
+    'POW': operator.pow
+}
+
+
 LO_AXIS_TO_STRING_CODE = {
     0: "X", 1: "Y", 2: "Z",
     3: "-X", 4: "-Y", 5: "-Z",
@@ -343,8 +373,8 @@ def rot_to(
     rotating_object,
     target_pos,
     front_axis_code,
-    speed,
-    time_per_frame
+    speed = 0,
+    time_per_frame = 0
 ):
     if rot_axis_index == 0:
         return xrot_to(
@@ -378,91 +408,20 @@ def rot_to(
 
 def load_json_as_dict(filepath):
     if not filepath.endswith('.json'):
-        path = f'{filepath}.json'
-    if path:
-        f = open(path, 'r')
+        filepath = f'{filepath}.json'
+    if filepath:
+        f = open(filepath, 'r')
         data = json.load(f)
         f.close()
         return data
+    else:
+        raise FileNotFoundError(f'File {filepath} could not be opened!')
 
 
 ###############################################################################
 # SCENE
 ###############################################################################
 
-def make_unique_light(old_lamp_ge: GameObject) -> GameObject:
-    '''TODO: Documentation
-    '''
-    scene = logic.getCurrentScene()
-    name = old_lamp_ge.name
-    old_lamp = old_lamp_ge.blenderObject
-    old_lamp.name = 'OLD_LAMP'
-
-    settings = [
-        'color',
-        'energy',
-        'diffuse_factor',
-        'specular_factor',
-        'volume_factor',
-        'shadow_soft_size',
-        'use_custom_distance',
-        'cutoff_distance',
-        'spot_size',
-        'spot_blend',
-        'show_cone',
-        'angle',
-        'shape',
-        'size',
-        'size_y',
-        'use_shadow',
-        'shadow_buffer_clip_start',
-        'shadow_buffer_bias',
-        'use_soft_shadows',
-        'use_contact_shadow',
-        'contact_shadow_distance',
-        'contact_shadow_bias',
-        'contact_shadow_thickness',
-        'shadow_cascade_count',
-        'shadow_cascade_fade',
-        'shadow_cascade_max_distance',
-        'shadow_cascade_exponent'
-    ]
-
-    types = {
-        'POINT': 'Point',
-        'AREA': 'Area',
-        'SPOT': 'Spot',
-        'SUN': 'Sun'
-    }
-
-    light_type = old_lamp.data.type
-    bpy.ops.object.light_add(
-        type=light_type,
-        location=old_lamp_ge.worldPosition,
-        rotation=old_lamp_ge.worldOrientation.to_euler()
-    )
-    index = 1
-    light = None
-    while light is None:
-        if types[light_type] in bpy.data.objects[-index].name:
-            light = bpy.data.objects[-index]
-        index += 1
-    for attr in settings:
-        try:
-            setattr(light.data, attr, getattr(old_lamp.data, attr))
-        except Exception:
-            pass
-    light.name = name
-    new_lamp_ge = scene.convertBlenderObject(light)
-    for p in old_lamp_ge.getPropertyNames():
-        new_lamp_ge[p] = old_lamp_ge[p]
-    old_lamp_ge.endObject()
-    real_name = light.name
-    light.name = real_name
-    if old_lamp_ge.parent:
-        new_lamp_ge.setParent(old_lamp_ge.parent)
-    return new_lamp_ge
-
 
 def get_closest_instance(game_obj: GameObject, name: str):
     '''TODO: Documentation
@@ -478,7 +437,25 @@ def get_closest_instance(game_obj: GameObject, name: str):
 
 
 def is_water(game_object: GameObject):
-    return WATER in game_object
+    return WATER in game_object.getPropertyNames()
+
+
+def get_child_by_name(obj, child, recursive=True, partial=False):
+    children = obj.childrenRecursive if recursive else obj.children
+    if partial:
+        for c in children:
+            if child in c.name:
+                return c
+    else:
+        return children.get(child)
+
+
+def check_vr_session_status() -> tuple[Vector, Matrix]:
+    """Get the current position and orientation of connected VR headset.
+    :returns: `tuple` of (`Vector`, `Matrix`)
+    """
+    session = bpy.context.window_manager.xr_session_state
+    return session is not None
 
 
 ###############################################################################
@@ -486,7 +463,7 @@ def is_water(game_object: GameObject):
 ###############################################################################
 
 
-def clamp(value: float, min: float = 0, max: float = 1) -> float:
+def clamp(value: float or Vector, min: float = 0, max: float = 1) -> float:
     """Clamp a value in between two other values.
 
     :param value: input value
@@ -495,7 +472,8 @@ def clamp(value: float, min: float = 0, max: float = 1) -> float:
 
     :returns: clamped value as float
     """
-
+    if isinstance(value, Vector):
+        return vec_clamp(value, min, max)
     if value < min:
         return min
     if value > max:
@@ -513,7 +491,6 @@ def vec_clamp(vec: Vector, min: float = 0, max: float = 1) -> Vector:
     :returns: clamped vector as float
     """
     vec = vec.copy()
-
     if vec.length < min:
         vec.normalize()
         return vec * min
@@ -575,7 +552,7 @@ def get_angle(a: Vector, b: Vector, up=Vector((0, 0, 1))) -> float:
 
     :returns: calculated value as float
     """
-    direction = get_direction(a, b)
+    direction = get_direction(Vector(a), Vector(b))
     rad: float = direction.angle(up)
     deg: float = rad * 180/math.pi
     return deg
@@ -613,3 +590,34 @@ def get_direction(a, b, local=False) -> Vector:
     d.normalize()
     return d
 
+
+def map_range(value, in_min, in_max, out_min, out_max, clamp=(None, None)):
+    result = (value - in_min) * (out_max - out_min) / (in_max - in_min) + out_min
+    if clamp[0] is not None and result < clamp[0]:
+        return clamp[0]
+    if clamp[1] is not None and result > clamp[1]:
+        return clamp[1]
+    return result
+
+def screen_to_world(x=None, y=None, distance=10):
+    camera = logic.getCurrentScene().active_camera
+    mouse = logic.mouse
+    x = x if x else mouse.position[0]
+    y = y if y else mouse.position[1]
+    direction = camera.getScreenVect(x, y)
+    origin = camera.worldPosition
+    aim = direction * -distance
+    return origin + (aim)
+
+def mouse_over(game_object: GameObject):
+    scene = game_object.scene
+    camera = scene.active_camera
+    distance = 2.0 * camera.getDistanceTo(game_object)
+    mouse = logic.mouse
+    mouse_position = mouse.position
+    target = camera.getScreenRay(
+        mouse_position[0],
+        mouse_position[1],
+        distance
+    )
+    return target is game_object
\ No newline at end of file
